@@ -19,15 +19,18 @@
 
 import copy
 import enum
+import operator
 import sys
 import textwrap
 import unittest
 
 from turberfield.dialogue.directives import Entity
-from turberfield.dialogue.types import EnumFactory
-from turberfield.dialogue.types import Player
 from turberfield.dialogue.model import Model
 from turberfield.dialogue.model import SceneScript
+from turberfield.dialogue.performer import Performer
+from turberfield.dialogue.types import EnumFactory
+from turberfield.dialogue.types import Stateful
+from turberfield.dialogue.types import Player
 
 
 class PropertyDirectiveTests(unittest.TestCase):
@@ -35,7 +38,8 @@ class PropertyDirectiveTests(unittest.TestCase):
     personae = [
         Player(name="Prof William Fuzzer Q.A Testfixture"),
         Player(name="Ms Anna Conda"),
-        Player(name="A Big Hammer")
+        Player(name="A Big Hammer"),
+        object()
     ]
 
     def test_property_getter(self):
@@ -148,9 +152,160 @@ class PropertyDirectiveTests(unittest.TestCase):
         self.assertEqual("state", p.attr)
         self.assertEqual(3, p.val)
 
+    def test_property_setter_bad_substitution(self):
+        content = textwrap.dedent(
+            """
+            .. entity:: P
+
+            Scene
+            ~~~~~
+
+            Shot
+            ----
+
+            .. property:: P.state |S_ID|
+
+            """)
+        ensemble = copy.deepcopy(PropertyDirectiveTests.personae)
+        script = SceneScript("inline", doc=SceneScript.read(content))
+        script.cast(script.select([ensemble[0]]))
+        model = script.run()
+        p = next(l for s, l in model if isinstance(l, Model.Property))
+        self.assertEqual("state", p.attr)
+        self.assertEqual(None, p.val)
+
+    def test_property_setter_good_substitution(self):
+        content = textwrap.dedent(
+            """
+            .. entity:: P
+            .. entity:: S
+
+            .. |S_ID| property:: S.id.int
+
+            Scene
+            ~~~~~
+
+            Shot
+            ----
+
+            .. property:: P.state |S_ID|
+
+            """)
+        ensemble = copy.deepcopy(PropertyDirectiveTests.personae)
+        script = SceneScript("inline", doc=SceneScript.read(content))
+        script.cast(script.select(ensemble))
+        model = script.run()
+        p = [l for s, l in model if isinstance(l, Model.Property)][-1]
+        self.assertEqual("state", p.attr)
+        self.assertIsInstance(p.val, int)
+
+
+@unittest.skipIf(
+    "discover" in sys.argv,
+    ("Testing condition directive: "
+    "Needs ~/py3.5/bin/python -m unittest turberfield.dialogue.test.test_model")
+)
+class ConditionDirectiveTests(unittest.TestCase):
+
+    class Rain(Stateful): pass
+    class Sleet(Stateful): pass
+    class Snow(Stateful): pass
+
+    class Weather(EnumFactory, enum.Enum):
+        quiet = 0
+        stormy = 1
+
+    content = textwrap.dedent(
+        """
+        .. entity:: WEATHER
+           :types: turberfield.dialogue.test.test_model.ConditionDirectiveTests.Rain
+                   turberfield.dialogue.test.test_model.ConditionDirectiveTests.Snow
+           :states: turberfield.dialogue.test.test_model.ConditionDirectiveTests.Weather.stormy
+
+        A stormy night
+        ~~~~~~~~~~~~~~
+
+        Outside.
+
+        Summary
+        -------
+
+        .. condition:: WEATHER.state
+                       turberfield.dialogue.test.test_model.ConditionDirectiveTests.Weather.stormy
+
+        [WEATHER]_
+
+            It's stormy!
+
+        Snow storm
+        ----------
+
+        .. condition:: WEATHER.__class__
+                       turberfield.dialogue.test.test_model.ConditionDirectiveTests.Snow
+
+        [WEATHER]_
+
+            Flurry, flurry.
+
+        Rainfall
+        --------
+
+        .. condition:: WEATHER.__class__
+                       turberfield.dialogue.test.test_model.ConditionDirectiveTests.Rain
+
+        [WEATHER]_
+
+            Pitter patter.
+        """)
+
+    effects = [
+        Rain().set_state(Weather.stormy),
+        Sleet().set_state(Weather.stormy),
+        Snow().set_state(Weather.quiet),
+    ]
+
+    def test_condition_evaluation_one(self):
+        effects = [
+            ConditionDirectiveTests.Rain().set_state(ConditionDirectiveTests.Weather.stormy),
+            ConditionDirectiveTests.Sleet().set_state(ConditionDirectiveTests.Weather.stormy),
+            ConditionDirectiveTests.Snow().set_state(ConditionDirectiveTests.Weather.quiet),
+        ]
+
+        script = SceneScript("inline", doc=SceneScript.read(self.content))
+        selection = script.select(effects)
+        self.assertTrue(all(selection.values()))
+        script.cast(selection)
+        model = script.run()
+        conditions = [l for s, l in model if isinstance(l, Model.Condition)]
+        self.assertEqual(3, len(conditions))
+
+        self.assertTrue(Performer.allows(conditions[0]))
+        self.assertFalse(Performer.allows(conditions[1]))
+        self.assertTrue(Performer.allows(conditions[2]))
+
+    def test_condition_evaluation_two(self):
+        effects = [
+            ConditionDirectiveTests.Rain().set_state(ConditionDirectiveTests.Weather.quiet),
+            ConditionDirectiveTests.Sleet().set_state(ConditionDirectiveTests.Weather.stormy),
+            ConditionDirectiveTests.Snow().set_state(ConditionDirectiveTests.Weather.stormy),
+        ]
+
+        script = SceneScript("inline", doc=SceneScript.read(self.content))
+        selection = script.select(effects)
+        self.assertTrue(all(selection.values()))
+        script.cast(selection)
+        model = script.run()
+        conditions = [l for s, l in model if isinstance(l, Model.Condition)]
+        self.assertEqual(3, len(conditions))
+
+        self.assertTrue(Performer.allows(conditions[0]))
+        self.assertTrue(Performer.allows(conditions[1]))
+        self.assertFalse(Performer.allows(conditions[2]))
+
+
 class FXDirectiveTests(unittest.TestCase):
 
-    def test_fx(self):
+    def test_fx_audio(self):
         content = textwrap.dedent(
             """
             Scene
@@ -159,7 +314,69 @@ class FXDirectiveTests(unittest.TestCase):
             Shot
             ----
 
-            .. fx:: turberfield.dialogue.sequences.battle_royal whack.wav
+            .. fx:: turberfield.dialogue.sequences.battle_royal whack.{0}
+               :offset: 0
+               :duration: 3000
+               :loop: 1
+
+            """)
+        for suffix in ("mp3", "ogg", "wav"):
+            with self.subTest(suffix=suffix):
+                text = content.format(suffix)
+                script = SceneScript("inline", doc=SceneScript.read(text))
+                model = script.run()
+                shot, cue = next(iter(model))
+                self.assertIsInstance(cue, Model.Audio)
+                self.assertEqual(
+                    "turberfield.dialogue.sequences.battle_royal",
+                    cue.package
+                )
+                self.assertEqual("whack.{0}".format(suffix), cue.resource)
+                self.assertEqual(0, cue.offset)
+                self.assertEqual(3000, cue.duration)
+                self.assertEqual(1, cue.loop)
+
+    def test_fx_image(self):
+        content = textwrap.dedent(
+            """
+            Scene
+            ~~~~~
+
+            Shot
+            ----
+
+            .. fx:: turberfield.dialogue.sequences.battle_royal whack.{0}
+               :offset: 0
+               :duration: 3000
+               :loop: 1
+
+            """)
+        for suffix in ("jpg", "jpeg", "png"):
+            with self.subTest(suffix=suffix):
+                text = content.format(suffix)
+                script = SceneScript("inline", doc=SceneScript.read(text))
+                model = script.run()
+                shot, cue = next(iter(model))
+                self.assertIsInstance(cue, Model.Still)
+                self.assertEqual(
+                    "turberfield.dialogue.sequences.battle_royal",
+                    cue.package
+                )
+                self.assertEqual("whack.{0}".format(suffix), cue.resource)
+                self.assertEqual(0, cue.offset)
+                self.assertEqual(3000, cue.duration)
+                self.assertEqual(1, cue.loop)
+
+    def test_fx_unhandled(self):
+        content = textwrap.dedent(
+            """
+            Scene
+            ~~~~~
+
+            Shot
+            ----
+
+            .. fx:: turberfield.dialogue.sequences.battle_royal whack.bin
                :offset: 0
                :duration: 3000
                :loop: 1
@@ -167,15 +384,26 @@ class FXDirectiveTests(unittest.TestCase):
             """)
         script = SceneScript("inline", doc=SceneScript.read(content))
         model = script.run()
-        shot, cue = next(iter(model))
-        self.assertEqual(
-            "turberfield.dialogue.sequences.battle_royal",
-            cue.package
-        )
-        self.assertEqual("whack.wav", cue.resource)
-        self.assertEqual(0, cue.offset)
-        self.assertEqual(3000, cue.duration)
-        self.assertEqual(1, cue.loop)
+        self.assertRaises(StopIteration, next, iter(model))
+
+    def test_fx_unknown(self):
+        content = textwrap.dedent(
+            """
+            Scene
+            ~~~~~
+
+            Shot
+            ----
+
+            .. fx:: turberfield.dialogue.sequences.battle_royal whack.nomime
+               :offset: 0
+               :duration: 3000
+               :loop: 1
+
+            """)
+        script = SceneScript("inline", doc=SceneScript.read(content))
+        model = script.run()
+        self.assertRaises(StopIteration, next, iter(model))
 
     def test_fx_bad_substitution(self):
         content = textwrap.dedent(
@@ -239,6 +467,39 @@ class FXDirectiveTests(unittest.TestCase):
         self.assertEqual(3000, cue.duration)
         self.assertEqual(1, cue.loop)
 
+    def test_fx_label_substitution(self):
+        content = textwrap.dedent(
+            """
+            .. entity:: P
+
+            Scene
+            ~~~~~
+
+            Shot
+            ----
+
+            .. fx:: turberfield.dialogue.sequences.battle_royal faces/|P_NAME|/surprise.png
+               :offset: 0
+               :duration: 3000
+               :label: |P_NAME| looks surprised
+
+            .. |P_NAME| property:: P.name.firstname
+            """)
+        ensemble = copy.deepcopy(PropertyDirectiveTests.personae)
+        script = SceneScript("inline", doc=SceneScript.read(content))
+        script.cast(script.select([ensemble[0]]))
+        model = script.run()
+        shot, cue = next(iter(model))
+        self.assertEqual(
+            "turberfield.dialogue.sequences.battle_royal",
+            cue.package
+        )
+        self.assertEqual("faces/William/surprise.png", cue.resource)
+        self.assertEqual(0, cue.offset)
+        self.assertEqual(3000, cue.duration)
+        self.assertEqual("William looks surprised", cue.label)
+
+
 class SelectTests(unittest.TestCase):
 
     @enum.unique
